package v1_0

import (
	"fmt"
	"net/http"
	"time"

	"adm-gateway/errors"
	model_v1 "adm-gateway/model/v1"
	param_v1 "adm-gateway/param/v1"
	"adm-gateway/parser"
	"adm-gateway/utils"

	"github.com/gin-gonic/gin"
	"git.getensh.com/common/gopkgsv2/jwtwrapper"
	"git.getensh.com/common/gopkgsv2/logger"
	"git.getensh.com/common/gopkgsv2/tasker/httptasker"
	"git.getensh.com/common/gopkgsv2/util"
	"go.uber.org/zap"
	"gorm.io/gorm"
)

// Login godoc
// @Summary 登陆
// @Description 登陆
// @Tags login,v1.0
// @Accept  json
// @Produce  json
// @Param body body v1.LoginBody true "body"
// @Success 200 {object} v1.LoginResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/login [post]
func (c *Controller) Login(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.LoginRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.LoginBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.LoginResponse{}

		// 读数据
		var err error
		res, err := model_v1.NewUserModel().Info(req.UserName)
		if err != nil && err != gorm.ErrRecordNotFound {
			return errors.SystemError
		}

		if res.Password != utils.MD5(req.Password) {
			return errors.PassWordError
		}

		if res.Status != 1 {
			return errors.UserStatusError
		}

		// 生成token
		token, err := jwtwrapper.GenToken(fmt.Sprintf("%d", res.ID), parser.Conf.Jwt.Issuer, `{}`,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			s, _ := json.MarshalToString(res.ID)
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", s),
				zap.String("error", err.Error()))
			return errors.SystemError
		}

		resp = &param_v1.LoginResponse{
			Data: param_v1.LoginData{
				Token: token,
			},
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
