package v1_0

import (
	"net/http"

	"adm-gateway/errors"
	param_v1 "adm-gateway/param/v1"
	"adm-gateway/pb"
	v1 "adm-gateway/pb/v1"

	"github.com/gin-gonic/gin"
	"git.getensh.com/common/gopkgsv2/logger"
	"git.getensh.com/common/gopkgsv2/tasker/httptasker"
	"git.getensh.com/common/gopkgsv2/util"
	"go.uber.org/zap"
)

// Maintainace godoc
// @Summary 维保手册
// @Description 维保手册
// @Tags maintainace,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param style_id query string false "style_id"
// @Success 200 {object} param_v1.GetMaintainaceManualResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/maintainace/manual [get]
func (c *Controller) GetMaintainaceManual(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GetMaintainaceManualRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.GetMaintainaceManualQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetMaintainaceManualResponse{}

		r := v1.MaintainManualRequest{
			StyleId: req.StyleId,
		}

		reply, err := pb.VehicleStyle.MaintainManual(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.MaintainManual"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary 维保项目
// @Description 维保项目
// @Tags maintainace,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @param item_type query int64 true "item_type"
// @Param style_id query string true "style_id"
// @Success 200 {object} param_v1.GetItemListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/maintainace/item_list [get]
func (c *Controller) GetItemList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GetItemListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.GetItemListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetItemListResponse{}

		r := v1.ItemListRequest{
			ItemType: req.ItemType,
			StyleId:  req.StyleId,
		}

		reply, err := pb.VehicleStyle.ItemList(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.ItemList"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary 更新维保手册
// @Description 更新维保手册
// @Tags maintainace,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param body body param_v1.UpdateMaintainManualBody true "body"
// @Success 200 {object} param_v1.UpdateMaintainManualResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/maintainace/manual [put]
func (c *Controller) UpdateManual(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.UpdateMaintainManualRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.UpdateMaintainManualBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.UpdateMaintainManualResponse{}

		r := v1.UpdateMaintainManualRequest{
			Id:        req.Id,
			ItemId:    req.ItemId,
			StartMile: req.StartMile,
			StartDate: req.StartDate,
			MileCycle: req.MileCycle,
			DateCycle: req.DateCycle,
			StyleId:   req.StyleId,
		}

		_, err := pb.VehicleStyle.UpdateMaintainManual(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.UpdateMaintainManual"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary 维保手册详情
// @Description 维保手册详情
// @Tags maintainace,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param style_id query string false "style_id"
// @Success 200 {object} param_v1.GetMaintainaceManualInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/maintainace/info [get]
func (c *Controller) GetMaintainaceManualInfo(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GetMaintainaceManualInfoRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.GetMaintainaceManualQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetMaintainaceManualInfoResponse{}

		r := v1.MaintainManualInfoRequest{
			StyleId: req.StyleId,
		}

		reply, err := pb.VehicleStyle.MaintainManualInfo(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.MaintainManualInfo"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary 配件列表
// @Description 配件列表
// @Tags maintainace,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param style_id query string false "style_id"
// @Success 200 {object} param_v1.GetMaintainaceManualInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/maintainace/style_item [get]
func (c *Controller) GetStyleItem(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GetMaintainaceManualInfoRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.GetMaintainaceManualQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetStyleItemResponse{}

		r := v1.MaintainManualRequest{
			StyleId: req.StyleId,
		}

		reply, err := pb.VehicleStyle.StyleItem(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.StyleItem"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary C2List
// @Description C2List
// @Tags maintainace,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param style_id query string false "style_id"
// @Success 200 {object} param_v1.GetMaintainaceManualInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/maintainace/c2/list [get]
func (c *Controller) GetC2List(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GetC2ListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.GetC2ListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetC2ListResponse{}

		r := v1.C2ListRequest{
			StyleId: req.StyleId,
		}

		reply, err := pb.VehicleStyle.C2List(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.C2List"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary 更新配件信息
// @Description 更新配件信息
// @Tags maintainace,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param body body param_v1.UpdateStyleItemBody true "body"
// @Success 200 {object} param_v1.UpdateMaintainManualResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/maintainace/style_item [put]
func (c *Controller) UpdateStyleItem(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.UpdateStyleItemRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.UpdateStyleItemBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.UpdateMaintainManualResponse{}

		r := v1.UpdateStyleItemRequest{}
		r.List = make([]*v1.StyleItem, 0, len(req.List))
		for _, v := range req.List {
			r.List = append(r.List, &v1.StyleItem{
				Id:            v.Id,
				StyleId:       v.StyleId,
				C2Id:          v.C2Id,
				Useage:        v.Useage,
				Detail:        v.Detail,
				AttributeRule: v.AttributeRule,
			})
		}

		_, err := pb.VehicleStyle.UpdateStyleItem(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.UpdateStyleItem"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary 删除维保信息
// @Description 删除维保信息
// @Tags maintainace,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param id path int64 true "style:id"
// @Success 200 {object} param_v1.GetStyleInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/maintainace/manual/{id} [delete]
func (c *Controller) DeleteMaintainace(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.DeleteMaintainaceRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, &req.DeleteMaintainacePath, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetStyleInfoResponse{}

		r := v1.DeleteMaintainaceRequest{
			Id: req.Id,
		}

		_, err := pb.VehicleStyle.DeleteMaintainace(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.DeleteMaintainace"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary 删除配件信息
// @Description 删除配件信息
// @Tags maintainace,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param id path int64 true "style_item:id"
// @Success 200 {object} param_v1.GetStyleInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/maintainace/style_item/{id} [delete]
func (c *Controller) DeleteStyleItem(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.DeleteMaintainaceRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, &req.DeleteMaintainacePath, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetStyleInfoResponse{}

		r := v1.DeleteStyleItemRequest{
			Id: req.Id,
		}

		_, err := pb.VehicleStyle.DeleteStyleItem(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.DeleteStyleItem"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
