package v1_0

import (
	"net/http"

	"adm-gateway/errors"
	param_v1 "adm-gateway/param/v1"
	"adm-gateway/pb"
	v1 "adm-gateway/pb/v1"

	"github.com/gin-gonic/gin"
	"git.getensh.com/common/gopkgsv2/logger"
	"git.getensh.com/common/gopkgsv2/tasker/httptasker"
	"git.getensh.com/common/gopkgsv2/util"
	"go.uber.org/zap"
)

// StyleList godoc
// @Summary 车型列表
// @Description 车型列表
// @Tags style,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param brand_name query string false "品牌名称"
// @Param style_name query string false "车型名称"
// @Param series_name query string false "车系名称"
// @Param brand_id query string false "品牌id"
// @Param series_id query string false "车系id"
// @Param style_id query string false "车型id"
// @Param has_manual query int32 false "-1:无手册， 1:有手册"
// @Param status query int32 false "-1:下线 1:上线"
// @Param model_year query int32 false "年款"
// @Param maker query string false "厂商"
// @Param page query int32 false "页码"
// @Param page_size query int32 false "每页数量，默认10"
// @Success 200 {object} param_v1.GetStyleListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/style/ [get]
func (c *Controller) StyleList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GetStyleListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.GetStyleListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetStyleListResponse{}

		r := v1.StyleListRequest{
			BrandName:  req.BrandName,
			Status:     req.Status,
			Page:       req.Page,
			PageSize:   req.PageSize,
			HasManual:  req.HasManual,
			ModelYear:  int64(req.ModelYear),
			Maker:      req.Maker,
			SeriesName: req.SeriesName,
			StyleName:  req.StyleName,
			BrandId:    req.BrandId,
			SeriesId:   req.SeriesId,
			StyleId:    req.StyleId,
		}

		reply, err := pb.VehicleStyle.StyleList(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.StyleList"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		if reply == nil {
			reply.List = make([]*v1.StyleList, 0)
		}
		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// UpdateStyle godoc
// @Summary 更新车型信息
// @Description 更新车型信息
// @Tags style,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param body body param_v1.UpdateStyleBody true "body"
// @Success 200 {object} param_v1.SearchResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/style/ [put]
func (c *Controller) UpdateStyle(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.UpdateStyleRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.UpdateStyleBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.UpdateBrandResponse{}

		r := v1.UpdateStyleRequest{
			Id:        req.ID,
			StyleName: req.StyleName,
			Status:    req.Status,
		}

		_, err := pb.VehicleStyle.UpdateStyle(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.UpdateStyle"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 车辆配置 godoc
// @Summary 获取车辆配置信息
// @Description 获取车辆配置信息
// @Tags style,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param id path int64 true "manual:id"
// @Success 200 {object} param_v1.GetStyleInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/style/config/{id} [get]
func (c *Controller) StyleInfo(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GetStyleInfoRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, &req.GetStyleInfoPath, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetStyleInfoResponse{}

		r := v1.StyleInfoRequest{
			Id: req.ID,
		}

		reply, err := pb.VehicleStyle.StyleInfo(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.StyleInfo"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		if reply.Aircondition == nil {
			reply.Aircondition = &v1.StyleInfoReply_Aircondition{}
		}
		if reply.Basic == nil {
			reply.Basic = &v1.StyleInfoReply_Basic{}
		}
		if reply.Body == nil {
			reply.Body = &v1.StyleInfoReply_Body{}
		}
		if reply.Chassis == nil {
			reply.Chassis = &v1.StyleInfoReply_Chassis{}
		}
		if reply.Control == nil {
			reply.Control = &v1.StyleInfoReply_Control{}
		}
		if reply.Electric == nil {
			reply.Electric = &v1.StyleInfoReply_Electric{}
		}
		if reply.Engine == nil {
			reply.Engine = &v1.StyleInfoReply_Engine{}
		}
		if reply.External == nil {
			reply.External = &v1.StyleInfoReply_External{}
		}
		if reply.Gearbox == nil {
			reply.Gearbox = &v1.StyleInfoReply_Gearbox{}
		}
		if reply.Intelligent == nil {
			reply.Intelligent = &v1.StyleInfoReply_Intelligent{}
		}
		if reply.Internal == nil {
			reply.Internal = &v1.StyleInfoReply_Internal{}
		}
		if reply.Light == nil {
			reply.Light = &v1.StyleInfoReply_Light{}
		}
		if reply.Media == nil {
			reply.Media = &v1.StyleInfoReply_Media{}
		}
		if reply.Optional == nil {
			reply.Optional = &v1.StyleInfoReply_Optional{}
		}
		if reply.Rearview == nil {
			reply.Rearview = &v1.StyleInfoReply_Rearview{}
		}
		if reply.Safety == nil {
			reply.Safety = &v1.StyleInfoReply_Safety{}
		}
		if reply.Seat == nil {
			reply.Seat = &v1.StyleInfoReply_Seat{}
		}
		if reply.Wheelbrake == nil {
			reply.Wheelbrake = &v1.StyleInfoReply_Wheelbrake{}
		}

		resp.Data = reply

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// UpdateStyleConfig godoc
// @Summary 更新车辆配置信息
// @Description 更新车辆配置信息
// @Tags style,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param body body param_v1.UpdateStyleConfigBody true "body"
// @Success 200 {object} param_v1.SearchResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/style/config [put]
func (c *Controller) UpdateStyleConfig(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.UpdateStyleConfigRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.UpdateStyleConfigBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.UpdateBrandResponse{}

		r := v1.UpdateStyleInfoRequest{
			Id:   req.ID,
			Data: req.Data,
		}

		_, err := pb.VehicleStyle.UpdateStyleInfo(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.UpdateStyleInfo"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// ExportStyle godoc
// @Summary 导出车型列表
// @Description 导出车型列表
// @Tags style,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param task_name query string false "任务名称"
// @Param fields query string false "字段用,分割"
// @Success 200 {object} param_v1.GetStyleListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/style/export [get]
func (c *Controller) ExportStyle(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ExportStyleListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.ExportStyleListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetStyleListResponse{}

		r := v1.ExportStyleRequest{
			TaskName: req.TaskName,
			Fields:   req.Fields,
		}

		_, err := pb.VehicleStyle.ExportStyle(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.ExportStyle"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// ExportStyleTaskList godoc
// @Summary 导出车型任务列表
// @Description 导出车型任务列表
// @Tags style,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Success 200 {object} param_v1.ExportStyleTaskListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/style/export_task [get]
func (c *Controller) ExportStyleTaskList(ctx *gin.Context) {
	// 解析参数
	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.ExportStyleTaskListResponse{}

		r := v1.EmptyReply{}

		reply, err := pb.VehicleStyle.ExportStyleTaskList(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.ExportStyleTaskList"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		resp.Data = reply
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.ExportStyleTaskList, 0)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, handleServiceTask)
}

// DeleteExportStyleTask godoc
// @Summary 删除导出车型任务
// @Description 删除导出车型任务
// @Tags style,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param id path int64 true "id"
// @Success 200 {object} param_v1.GetStyleInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/style/export_task/{id} [delete]
func (c *Controller) DeleteExportStyleTask(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.DeleteExportStyleTaskRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, &req.DeleteMaintainacePath, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetStyleInfoResponse{}

		r := v1.DeleteExportStyleTaskRequest{
			Id: req.Id,
		}

		_, err := pb.VehicleStyle.DeleteExportStyleTask(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.DeleteExportStyleTask"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
