package v1_0

import (
	"adm-gateway/errors"
	param_v1 "adm-gateway/param/v1"
	"adm-gateway/pb"
	v1 "adm-gateway/pb/v1"
	"net/http"

	"github.com/gin-gonic/gin"
	"git.getensh.com/common/gopkgsv2/logger"
	"git.getensh.com/common/gopkgsv2/tasker/httptasker"
	"git.getensh.com/common/gopkgsv2/util"
	"go.uber.org/zap"
)

// MaintenanceTemplate godoc
// @Summary 模板名称列表
// @Description 模板名称列表
// @Tags template_list,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param name query string false "名称"
// @Success 200 {object} param_v1.TemplateListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/template/list [get]
func (c *Controller) TemplateList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.TemplateListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.TemplateListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.TemplateListResponse{}

		r := v1.TemplateListRequest{
			Name: req.Name,
		}

		reply, err := pb.VehicleStyle.TemplateList(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.VehicleStyle.TemplateList"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		if reply.List == nil {
			reply.List = make([]*v1.Template, 0)
		}
		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// MaintenanceTemplate godoc
// @Summary 模板列表
// @Description 模板列表
// @Tags template_list,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param tid query int64 false "tid"
// @Param page query int32 false "页码"
// @Param page_size query int32 false "每页数量，默认10"
// @Success 200 {object} param_v1.TemplateListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/template/detail/list [get]
func (c *Controller) TemplateDetailList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.TemplateDetailListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.TemplateDetailListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.TemplateDetailListResponse{}

		r := v1.TemplateDetailListRequest{
			Tid:      req.Tid,
			PageSize: req.PageSize,
			Page:     req.Page,
		}

		reply, err := pb.VehicleStyle.TemplateDetailList(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.VehicleStyle.TemplateDetailList"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		if reply.List == nil {
			reply.List = make([]*v1.TemplateDetail, 0)
		}
		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// MaintenanceTemplate godoc
// @Summary 模板详情
// @Description 模板详情
// @Tags template_list,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param tid query int64 false "tid"
// @Success 200 {object} param_v1.TemplateListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/template/detail [get]
func (c *Controller) TemplateDetail(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.TemplateDetailRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.TemplateDetailQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.TemplateDetailResponse{}

		r := v1.TemplateDetailRequest{
			Tid: req.Tid,
		}

		reply, err := pb.VehicleStyle.TemplateDetail(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.VehicleStyle.TemplateDetail"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// MaintenanceTemplate godoc
// @Summary 新增模板
// @Description 新增模板
// @Tags template_list,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param body body param_v1.AddTemplateDetailBody true "body"
// @Success 200 {object} param_v1.TemplateListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/template/detail [post]
func (c *Controller) AddTemplateDetail(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.AddTemplateDetailRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.AddTemplateDetailBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.AddTemplateDetailResponse{}

		r := v1.AddTemplateDetailRequest{
			Tid:       req.Tid,
			ItemId:    req.ItemId,
			StartMile: req.StartMile,
			StartDate: req.StartDate,
			MileCycle: req.MileCycle,
			DateCycle: req.DateCycle,
			Name:      req.Name,
		}

		reply, err := pb.VehicleStyle.AddTemplateDetail(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.VehicleStyle.AddTemplateDetail"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary 更新维保模板
// @Description 更新维保模板
// @Tags template_list,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param body body param_v1.UpdateMaintainManualBody true "body"
// @Success 200 {object} param_v1.UpdateMaintainManualResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/template/detail [put]
func (c *Controller) UpdateTemplateDetail(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.UpdateMaintainManualRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.UpdateMaintainManualBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.UpdateMaintainManualResponse{}

		r := v1.UpdateMaintainManualRequest{
			Id:        req.Id,
			ItemId:    req.ItemId,
			StartMile: req.StartMile,
			StartDate: req.StartDate,
			MileCycle: req.MileCycle,
			DateCycle: req.DateCycle,
			Name:      req.Name,
			Tid:       req.Tid,
		}

		_, err := pb.VehicleStyle.UpdateTemplateDetail(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.UpdateTemplateDetail"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary 选择维保模板更新到车型维保手册
// @Description 选择维保模板更新到车型维保手册
// @Tags template_list,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @Param body body param_v1.ChooseTemplateDetailBody true "body"
// @Success 200 {object} param_v1.UpdateMaintainManualResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/template/detail/choose [post]
func (c *Controller) ChooseTemplateDetail(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ChooseTemplateDetailRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.ChooseTemplateDetailBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.UpdateMaintainManualResponse{}

		r := v1.ChooseTemplateDetailRequest{
			Tid:     req.Tid,
			StyleId: req.StyleId,
		}

		_, err := pb.VehicleStyle.ChooseTemplateDetail(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.ChooseTemplateDetail"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// Maintainace godoc
// @Summary 维保项目
// @Description 维保项目
// @Tags template_list,v1.0
// @Accept  json
// @Produce  json
// @Param token header string true "jwt token"
// @param item_type query int64 true "item_type"
// @param tid query int64 false "template_id"
// @Success 200 {object} param_v1.GetItemListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1.0/template/item_list [get]
func (c *Controller) GetTemplateItemList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GetTemplateItemListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.GetTemplateItemListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := &param_v1.GetItemListResponse{}

		r := v1.TemplateItemListRequest{
			ItemType: req.ItemType,
			Tid:      req.Tid,
		}

		reply, err := pb.VehicleStyle.TemplateItemList(ctx.Request.Context(), &r)
		if err != nil {
			s, _ := json.MarshalToString(r)
			logger.Error("func",
				zap.String("call", "pb.vehicleStyle.TemplateItemList"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			return errors.ErrorTransForm(err)
		}

		resp.Data = reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
